@@ -1,6 +1,6 @@
 package Protocol::SPDY::Constants;
 {
-  $Protocol::SPDY::Constants::VERSION = '0.999_002';
+  $Protocol::SPDY::Constants::VERSION = '0.999_003';
 }
 use strict;
 use warnings;
@@ -12,7 +12,7 @@ Protocol::SPDY::Constants - constant definitions for the SPDY protocol
 
 =head1 VERSION
 
-version 0.999_002
+version 0.999_003
 
 =head1 SYNOPSIS
 
@@ -22,6 +22,70 @@ version 0.999_002
 
 Provides some constants.
 
+=head1 CONSTANTS
+
+=head2 FLAG_FIN
+
+Value for the FIN flag in control/data frames.
+
+=head2 FLAG_COMPRESS
+
+Compression flag - currently unused
+
+=head2 FLAG_UNI
+
+Unidirectional flag - used to mark a stream as not requiring a reply.
+
+=head2 HEADER_LENGTH
+
+Number of bytes required in order to work out how big a frame will be.
+
+=head2 ZLIB_DICTIONARY
+
+Initial compression dictionary for the zlib compression protocol used
+for name/value pair data.
+
+=head2 MAX_SUPPORTED_VERSION
+
+Highest version we know about.
+
+=head1 RST_STREAM CODES
+
+The following codes can be returned as values in a
+L<RST_STREAM|Protocol::SPDY::Frame::Control::RST_STREAM> frame. See
+the L<status_code|Protocol::SPDY::Frame::Control::RST_STREAM/status_code>
+and L<status_code_as_text|Protocol::SPDY::Frame::Control::RST_STREAM/status_code_as_text>
+methods.
+
+=head2 PROTOCOL_ERROR
+
+There was a protocol violation in something we've sent to the other side.
+
+=head2 INVALID_STREAM
+
+The requested stream is not valid (for example, when we receive a reply
+to a stream that we did not initiate).
+
+=head2 REFUSED_STREAM
+
+We do not want this stream.
+
+=head2 UNSUPPORTED_VERSION
+
+The protocol version requested is not supported by this library.
+
+=head2 CANCEL
+
+Used by the initiator to stop an active stream before normal completion.
+
+=head2 INTERNAL_ERROR
+
+Generic error when internal state is invalid.
+
+=head2 FLOW_CONTROL_ERROR
+
+We violated the expected window update behaviour.
+
 =cut
 
 use constant {
@@ -149,15 +213,18 @@ use constant {
 		6 => 'PING',
 		7 => 'GOAWAY',
 		8 => 'HEADERS',
+		9 => 'WINDOW_UPDATE',
+		10 => 'CREDENTIAL',
 	},
 	SETTINGS_BY_ID => {
 		# Expected upload bandwidth
-		1 => 'UPLOAD_BANDWIDTH',
+		1 => 'EXPECTED_UPLOAD_BANDWIDTH',
 		# Expected download bandwidth
-		2 => 'DOWNLOAD_BANDWIDTH',
+		2 => 'EXPECTED_DOWNLOAD_BANDWIDTH',
 		# How long we expect packets to take to go from here to there and back again
-		3 => 'ROUND_TRIP_TIME',
-		# How many streams we want
+		3 => 'EXPECTED_ROUND_TRIP_TIME',
+		# How many streams we want - clients which do not want server push seem to
+		# use a 0 value here to disable the feature
 		4 => 'MAX_CONCURRENT_STREAMS',
 		# TCP initial client window size
 		5 => 'CURRENT_CWND',
@@ -183,8 +250,8 @@ use constant {
 
 # Reversed lookup mappings
 use constant {
-	FRAME_TYPE_BY_NAME => +{ reverse %{+FRAME_TYPE_BY_ID} },
-	SETTINGS_BY_NAME => +{ reverse %{+SETTINGS_BY_ID} },
+	FRAME_TYPE_BY_NAME      => +{ reverse %{+FRAME_TYPE_BY_ID} },
+	SETTINGS_BY_NAME        => +{ reverse %{+SETTINGS_BY_ID} },
 	RST_STATUS_CODE_BY_NAME => +{ reverse %{+RST_STATUS_CODE_BY_ID} },
 };
 
@@ -192,6 +259,7 @@ our @EXPORT_OK = qw(
 	FLAG_FIN FLAG_COMPRESS FLAG_UNI
 	FRAME_TYPE_BY_ID FRAME_TYPE_BY_NAME
 	SETTINGS_BY_ID SETTINGS_BY_NAME
+	FLAG_SETTINGS_PERSISTED FLAG_SETTINGS_PERSIST_VALUE
 	RST_STATUS_CODE_BY_ID RST_STATUS_CODE_BY_NAME
 	HEADER_LENGTH
 	ZLIB_DICTIONARY MAX_SUPPORTED_VERSION
